/*
  global variables
*/

var widgetID = (new Date()).getTime();
var command = null;
var debugMode = true;
var reloadCounter;
var lastLoaded = (new Date()).getTime();
var reloadLevel = 0;
var waitPeriod = [ 5, 10, 20, 60, 120, 240, 480, 720, 1440 ];
var timer = null;
var reservedSize = { width:300, height:300 };
var modules = new Array();
var localizedStrings = null;
var currentSkin = "default";
var minSize = { width:300, height:200 };
//var windowSize = null;

function tr( text ) {
    if ( localizedStrings ) {
        if ( localizedStrings[ text ] ) {
            return localizedStrings[ text ];
        }
    }
    return text;
}

// debug
function DEBUG( message ) {
    if ( !debugMode ) return;
    var elem = document.getElementById( 'debugDiv' );
    if ( elem ) {
        elem.innerText = message;
    }
}

function forceReload( event ) {
    reloadCounter = null;
    loadInformation( document.getElementById( "modules" ).value );
}

function tryReload() {
    var current = (new Date()).getTime();
    if ( reloadCounter == null ) {
        reloadLevel = 0;
        reloadCounter = current + waitPeriod[ 0 ] * 1000 * 60;
    }
    DEBUG( "trying reloading " + reloadCounter + " / " + current + " : " + ( ( reloadCounter - current ) / 1000 ) + " " + reloadLevel );

    if ( reloadCounter > current ) return;
    loadInformation( document.getElementById( "modules" ).value );
    reloadLevel ++;
    if ( reloadLevel >= waitPeriod.length ) {
        reloadLevel = waitPeriod.length - 1;
    }
    reloadCounter = current + waitPeriod[ reloadLevel ] * 1000 * 60;
    
}

function stripItem( str ) {
    var start = 0;
    var end = str.length;
    if ( str.length < 2 ) return str;
    if ( str.charAt( 0 ) == '"' ) start = 1;
    if ( str.charAt( str.length - 1 ) == '"' ) end = str.length - 1;
    return str.substring( start, end );
}

function __strip_item( str ) {
    var start = 0;
    var end = str.length;
    if ( str.length < 2 ) return str;
    if ( str.charAt( 0 ) == '"' ) start = 1;
    if ( str.charAt( str.length - 1 ) == '"' ) end = str.length - 1;
    return str.substring( start, end );
}

function DSModule( line ) {
    this.available = false;
    this.isAvailable = DSModule__isAvailable;
    var items     = line.split( ',' );
    if ( items.length < 5 ) return;
    this.data     = line;
    this.name     = __strip_item( items[ 0 ] );
    this.label    = __strip_item( items[ 1 ] );
    this.sports   = __strip_item( items[ 2 ] );
    this.language = __strip_item( items[ 3 ] );
    this.country  = __strip_item( items[ 4 ] );
    this.available = true;
}

function DSModule__isAvailable() {
    return this.available;
}

function listModules( country, language, sports ) {
    var select = document.getElementById( "modules" );
    if ( select ) {
        select.innerHTML = "";
        if ( country == undefined || country == "all" ) country = null;
        if ( language == undefined || language == "all" ) language = null;
        if ( sports == undefined || sports == "all" ) sports = null;
        for ( var i = 0; i < modules.length; i++ ) {
            var module = modules[ i ];
            if ( country ) {
                if ( module.country != country ) continue;
            }
            if ( language ) {
                if ( module.language != language ) continue;
            }
            if ( sports ) {
                if ( module.sports != sports ) continue;
            }
            var option = document.createElement( 'option' );
            option.setAttribute( 'value', module.name );
            option.appendChild( document.createTextNode( module.label ) );
            select.appendChild( option );
        }
    }
}

function changeModuleList() {
    var country = null;
    var language = null;
    var sports = null;
    var skin = null;
    var elem_c = document.getElementById( "country" );
    var elem_l = document.getElementById( "language" );
    var elem_s = document.getElementById( "sports" );
//     var elem_a = document.getElementById( "skin" );
    if ( elem_c ) {
        country = elem_c.value;
    }
    if ( elem_l ) {
        language = elem_l.value;
    }
    if ( elem_s ) {
        sports = elem_s.value;
    }
//     if ( elem_a ) {
//         skin = elem_a.value;
//     }
    //DEBUG( country + " " + language + " " + sports );
    if ( window.widget ) {
        if ( country ) widget.setPreferenceForKey( country, 'dashsticker-country' );
        if ( language ) widget.setPreferenceForKey( language, 'dashsticker-language' );
        if ( sports ) widget.setPreferenceForKey( sports, 'dashsticker-sports' );
    }
    //DEBUG( country + " " + language + " " + sports + " " );
    listModules( country, language, sports );
}

function changeSkin( skin ) {
    if ( skin ) {
        currentSkin = skin;
    }

    var elems = [ "TopLeft","TopRight", "BottomLeft","BottomRight",
                  "LeftEdge","RightEdge","TopEdge","BottomEdge","Body"];

    DEBUG( currentSkin );

    for ( var i = 0; i < elems.length; i++ ) {
        var e = document.getElementById( elems[ i ] );
        if ( e ) {
            var url = "./skins/" + currentSkin+ "/" + elems[ i ] + ".png";
            e.style.backgroundImage ="url(" + url + ")";
        }
    }
    if ( window.widget ) {
        widget.setPreferenceForKey( skin, 'dashsticker-skin' );
    }
}

function listCountries() {
    listSelection( "country" );
}

function listLanguages() {
    listSelection( "language" );
}
function listSports() {
    listSelection( "sports" );
}

function listSkins() {
    var select = document.getElementById( 'skin' );
    if ( window.widget && select ) {
        var def_skin = widget.preferenceForKey( 'dashsticker-skin' );
        DEBUG( def_skin );
        var outputs = widget.system( "/usr/bin/python getskinlist.py", null ).outputString.split( "\n" );
        select.innerHTML = "";
        for ( var i = 0; i < outputs.length; i++ ) {
            if ( outputs[ i ].length == 0 ) continue;
            var option = document.createElement( 'option' );
            option.setAttribute( 'value', outputs[ i ] );
            if ( outputs[ i ] == def_skin ) {
                option.setAttribute( 'selected' );
                changeSkin( outputs[ i ] );
            }
            option.appendChild( document.createTextNode( tr( outputs[ i ] ) ) );
            select.appendChild( option );
        }
    }
}

function listSelection( category ) {
    var selected = "all";
    if ( window.widget ) {
        var pref = widget.preferenceForKey( "dashsticker-" + category );
        if ( pref && pref.length > 0 ) {
            selected = pref;
        }
    }

    var items = new Array();
    for ( var i = 0; i < modules.length; i++ ) {
        var item = modules[ i ][ category ];//.language;
        var flag_exists = false;
        for ( var j = 0; j < items.length; j++ ) {
            if ( items[ j ] == item ) {
                flag_exists = true;
                break;
            }
        }
        if ( !flag_exists ) {
            items.push( item );
        }
    }

    var elem_c = document.getElementById( category );
    if ( elem_c ) {
        elem_c.innerHTML = "<option value=\"all\">" + tr( "All" ) + "</option>";
        for ( var i = 0; i < items.length; i++ ) {
            var option = document.createElement( 'option' );
            option.setAttribute( 'value', items[ i ] );
            if ( items[ i ] == selected ) {
                option.setAttribute( 'selected' );
            }
            var text = tr( items[ i ] );
            option.appendChild( document.createTextNode( text ) );
            elem_c.appendChild( option );
        }
    }
}

function refreshModules( args ) {
    if ( command != null ) {
        var output = command.outputString;
        mlist = output.split( "\n" );
        var select = document.getElementById( "modules" );
        
        for ( var i = 0; i < mlist.length; i++ ) {
            var module = new DSModule( mlist[ i ] );
            if ( module.isAvailable() == false ) continue;
            modules.push( module );
        }
        listModules( null, null, null );
        listCountries();
        listLanguages();
        listSports();
        listSkins();
        changeModuleList();
        command = null;
    }
}

function setup() {
    if ( window.widget ) {
        command = widget.system( "/usr/bin/python loader.py", refreshModules );
        widget.onremove = cleanup;

        createGenericButton( document.getElementById( 'done' ), tr( "Done" ), hidePreference );

    }
    document.getElementById( 'label-country' ).innerText = tr( 'Country' );
    document.getElementById( 'label-language' ).innerText = tr( 'Language' );
    document.getElementById( 'label-sports' ).innerText = tr( 'Sports' );
    document.getElementById( 'label-skin' ).innerText = tr( 'Skin' );
    document.getElementById( 'label-module' ).innerText = tr( 'Module' );

//     var iframe = document.getElementById( 'score' );
//     var computedStyle = document.defaultView.getComputedStyle( iframe );
//     var width  = parseInt( computedStyle.getPropertyValue( "width" ) );
//     var height = parseInt( computedStyle.getPropertyValue( "height" ) );
//     windowSize = { width:width, height:height };
}

function endHandler(arg ) {
    var output = command.outputString;
//    DEBUG( output );
    var iframe = document.getElementById( "score" );
    if ( iframe ) {
        iframe.src = output;
//         var computedStyle = document.defaultView.getComputedStyle( iframe );
//         var width  = computedStyle.getPropertyValue( "width" );
//         var height = computedStyle.getPropertyValue( "height" );
//         DEBUG( "( " + width + ", " + height + " ) " );
    }
    command = null;
}

function changeModule(event) {
    var select = document.getElementById( "modules" );
    DEBUG( select.value );
}

// function changed(event,elem) {
//     load( event, elem );
// }

function loadInformation( modulename, skinname ) {
    var value = modulename;
    if ( skinname && skinname.length > 0 ) {
        currentSkin = skinname;
    } else {
        skinname = currentSkin;
    }
    if ( value == null || value == "" ) return;
    if ( window.widget ) {
        var pycommand = "/usr/bin/python loader.py \"" + value + "\" " + widgetID + " " + skinname;
        DEBUG( pycommand );
        if ( command != null ) command.cancel();
        var iframe = document.getElementById( 'score' );
        if ( iframe ) {
            iframe.src = "Loading.html";
        }

        command = widget.system( pycommand, endHandler );
    }
}

function cleanup() {
    if ( window.widget ) {
        widget.system( "/usr/bin/python cleanup.py " + widgetID, null );
    }
}




var lastPos; 
function resizeMouseDown(event) 
{ 
    var x = event.x + window.screenX; 
    var y = event.y + window.screenY; 
    document.addEventListener("mousemove", resizeMouseMove, true); 
    document.addEventListener("mouseup", resizeMouseUp, true); 
    lastPos = {x:x, y:y}; 
    event.stopPropagation(); 
    event.preventDefault(); 
} 

function resizeMouseMove(event) 
{ 
    var screenX = event.x + window.screenX; 
    var screenY = event.y + window.screenY; 
    var dx = 0;//screenX - lastPos.x;
    var dy = 0;//screenY - lastPos.y;
    if ( window.outerWidth + ( screenX - lastPos.x ) >= minSize.width ) {
        dx = screenX - lastPos.x;
        lastPos.x = screenX;
    }
    if ( window.outerHeight + ( screenY - lastPos.y ) >= minSize.height ) {
        dy = screenY - lastPos.y;
        lastPos.y = screenY;
    }

    window.resizeBy( dx, dy );
    updateCoordinates();

//     if ( windowSize.width + dx < minSize.width ) {
//         dx = minSize.width - windowSize.width;
//         screenX = lastPos.x + dx;//( minSize.width - windowSize.width ) / 2;//- ( screenX - lastPos.x );
//     }
//     if ( windowSize.height + dy < minSize.height ) {
//         dy = minsize.height - windowSize.height;
//         screenY = lastPos.y + dy;//( minSize.height - windowSize.height ) / 2;
//     }
//     windowSize.width += dx;
//     windowSize.height += dy;
//     DEBUG( " ( " + window.screenX + ", " + window.screenY + " ) - ( " + lastPos.x + ", " + lastPos.y + " ) " );
// //    DEBUG( "( " + windowSize.width + ", " + windowSize.height + " )" );
//     window.resizeBy( dx, dy );//screenX - lastPos.x, screenY - lastPos.y); 
//     //window.resizeBy(screenX - lastPos.x, screenY - lastPos.y); 
//     lastPos = {x:screenX, y:screenY}; 
    event.stopPropagation(); 
    event.preventDefault(); 
} 

function resizeMouseUp(event) 
{ 
    document.removeEventListener("mousemove", resizeMouseMove, true); 
    document.removeEventListener("mouseup", resizeMouseUp, true); 
    event.stopPropagation(); 
    event.preventDefault(); 
} 

function flipToPreferencePanel( event ) {
//function showPreference(event) {
//    DEBUG( "pref" );
    var front = document.getElementById( "front" ); 
    var back  = document.getElementById( "behind" ); 
    var computedStyle = document.defaultView.getComputedStyle( front );
    var resized = false;
    reservedSize.width  = parseInt( computedStyle.getPropertyValue( "width" ) );
    reservedSize.height = parseInt( computedStyle.getPropertyValue( "height" ) );
    //DEBUG( "reserved " + reservedSize.width + ", " + reservedSize.height + " / " + window.size );
    if ( isLarger( 300, 200, reservedSize.width, reservedSize.height ) ) {
        window.resizeTo( 300, 200 );
        resized = true;
    }
    if (window.widget) {
        widget.prepareForTransition("ToBack"); 
    }
    if ( !resized ) {
        window.resizeTo( 300, 200 );
        resized = true;
    }

    back.style.display  = "block"; 
    front.style.display = "none"; 

    if (window.widget) {
        setTimeout ('widget.performTransition();', 0);  
    }
}

function isLarger( w1, h1, w2, h2 ) {
    if ( w1 > w2 || h1 > h2 ) {
        return true;
    }
    return false;
}

function hidePreference( event ) { 
    var front = document.getElementById("front"); 
    var back = document.getElementById("behind"); 
    var computedStyle = document.defaultView.getComputedStyle( back );
    var resized = false;
    var currentSize = { width:parseInt( computedStyle.getPropertyValue( "width" ) ), height:parseInt( computedStyle.getPropertyValue( "height" ) ) };

    DEBUG( "revert to " + reservedSize.width + ", " + reservedSize.height );
//    DEBUG( reservedSize );
    if ( isLarger( reservedSize.width, reservedSize.height, currentSize.width, currentSize.height ) ) {
        window.resizeTo( reservedSize.width, reservedSize.height );
        resized = true;
    }
    if (window.widget) {
        widget.prepareForTransition("ToFront"); 
    }
    if ( resized == false ) {
        window.resizeTo( parseInt( reservedSize.width ), 
                         parseInt( reservedSize.height ) );
    }
    front.style.display="block"; 
    back.style.display="none"; 
    if (window.widget) {
        setTimeout ('widget.performTransition();', 0);  
    }
    var select = document.getElementById( "modules" );
    if ( select && select.value) {
        reloadCounter = null;
        if ( timer != null ) {
            clearInterval( timer );
            timer = null;
        }
        timer = setInterval( tryReload, 10000 );
        loadInformation( select.value );
    }
} 

function pushDoneButton( event ) {
}

function revertDoneButton( event ) {
}

function enterFlipIcon(event) {
    document.getElementById( "flip" ).style.display = "block";
//    DEBUG( "enter icon" );
}

function exitFlipIcon(event) {
//    DEBUG( "exit icon" );
    document.getElementById( "flip" ).style.display = "none";
//     document.getElementById( "flip" ).style.display = "none";
}

